/*!The Treasure Box Library
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2009-present, TBOOX Open Source Group.
 *
 * @author      ruki
 * @file        algorithm.h
 * @defgroup    algorithm
 *
 */
#ifndef TB_ALGORITHM_H
#define TB_ALGORITHM_H

/* //////////////////////////////////////////////////////////////////////////////////////
 * includes
 */
#include "prefix.h"
#include "predicate.h"
#include "distance.h"
#include "for.h"
#include "for_if.h"
#include "rfor.h"
#include "rfor_if.h"
#include "sort.h"
#include "heap_sort.h"
#include "quick_sort.h"
#include "insert_sort.h"
#include "bubble_sort.h"
#include "find.h"
#include "find_if.h"
#include "rfind.h"
#include "rfind_if.h"
#include "binary_find.h"
#include "binary_find_if.h"
#include "walk.h"
#include "rwalk.h"
#include "count.h"
#include "count_if.h"
#include "remove.h"
#include "remove_if.h"
#include "remove_first.h"
#include "remove_first_if.h"

#endif
